@@ -15,7 +15,7 @@
 #                                 #
 ###################################
 
-fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm",der=1,vfactor=15,iterlim=300,accuracy="standard",kmst=!dmm$st,kmrep=5,postst=TRUE) { 
+fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm",der=1,vfactor=15,iterlim=100,accuracy="standard",kmst=!dmm$st,kmrep=5,postst=TRUE) { 
 	mod=dmm #keep the original model
 ## create good starting values if not provdided
 	if(kmst) {
@@ -52,6 +52,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			dmm$st=TRUE
 		}
 	} # end else
+	
 	if(postst) {
 		if(class(dmm)[1]!="dmm") {
 			warning("No automatic boosting of starting values for other than single component 1-group models.")
@@ -106,6 +107,11 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		}
 	}
 	
+	# degrees of freedom in the data
+	nobs1=0
+	if(xgmod$ng==1) nobs1=sum(replicates(dat)*ntimes(dat))
+	else for(i in 1:xgmod$ng) nobs1 = nobs1 + sum(replicates(dat[[i]])*ntimes(dat[[i]]))
+	
 ## parameter reduction, selection etc	
 
 	# model has no td pars or it has td pars and they should be fitted
@@ -116,16 +122,16 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		bllin=xgmod$bllin
 		bu=xgmod$bu
 		bulin=xgmod$bulin
-		# make vector of pars that are fixed to zero
 		fixed=xgmod$fixed
 	} else {
 		# model has td pars but these should not be fitted
 		# split pars in normal pars and tdpars
 		fitpars=xgmod$pars[1:xgmod$npars]
 		# split A, bl/u, and bl/ulin accordingly
-		A=xgmod$A[,1:xgmod$npars]
+		A=xgmod$A[,1:xgmod$npars,drop=FALSE]
 		bl=xgmod$bl[1:xgmod$npars]
 		bu=xgmod$bu[1:xgmod$npars]
+		fixed=xgmod$fixed[1:xgmod$npars]
 		
 		# remove zero rows from A that result from removing tdpars, also remove rows with only one entry
 		idx2=numeric(0)
@@ -144,13 +150,13 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			bllin=xgmod$bllin
 			bulin=xgmod$bulin
 		}
-		fixed=xgmod$fixed[1:xgmod$npars]
 	}
 	
 	# check for zero rows in A which may result from fixing parameters at zero, and remove them
-	# set zeroes in A when pars are fixed, and change bllin and bulin accordingly]
-	# this is also done in function dmm, but users may add/change etc the linear constraint matrix 
+	# set zeroes in A when pars are fixed, and change bllin and bulin accordingly
+	# this is also done in function (mg-mix-)dmm, but users may add/change etc the linear constraint matrix 
 	# after defining the model
+	
 	if(nrow(A)>0) {
 		Bcomplete=matrix(as.logical(A),nr=nrow(A))
 		xcomplete=apply(Bcomplete,1,sum)
@@ -172,14 +178,15 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		bulin=bulin[x>1]
 	}
 	
-	# change pars, npars, A, bl, bu, bllin using fixed
+	# only non-fixed pars are passed the optimization routine
 	pars=fitpars[which(fixed==1)]
-	bu=bu[which(fixed==1)]
-	bl=bl[which(fixed==1)]
 	
-##  call npmain to optimize the model: non-linear constraints not implemented yet
+##  call npmain to optimize the model (non-linear constraints not implemented yet)
 	if(method=="npsol") {
 		
+		bu=bu[which(fixed==1)]
+		bl=bl[which(fixed==1)]
+		
 		if(printlevel>29) {
 			print(A)
 			print(bl)
@@ -199,10 +206,11 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		## npsol optimization options
 		itlim=paste(c("Iteration Limit = ", iterlim , "   "), collapse=" ")
 		npopts=c(itlim,
-			"Print level = 10 ",
+			"Print level = 20 ",
+			"Minor print level = 10",
 			"Derivative level = 0 ",
 			"Hessian = No ",
-			"Verify level = 3 ")
+			"Verify level = 0 ")
 		
 # 			"Optimality tolerance = 1.0e-12 ")
 		# verify options: 0 is good for standard markovian and latent class models
@@ -210,7 +218,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		
 		optset=lapply(npopts,FUN=optfor <- function(opt) {.Fortran("npoptn",as.character(opt),PACKAGE="depmix") } )
 		# optfile can be used to read in an optionsfile called npoptn from the working directory
-		optfile=0 # using this will overwrite above specified default options
+		optfile=0 # using this will override above specified default options
 		derivatives=der # are derivatives to be used or not
 		cj=1 # non linear constraints are not supported yet, hence cj=1
 		maxnpcalls=1 # maxnpcalls is not used at the moment
@@ -255,6 +263,154 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		z
 	}
 	
+	if(method=="gloptim") {
+		
+		# information: fitpars, fixed, bu, bl, A, bulin, bllin (fixed is redundant ...)
+		# this is assuming that A has no redundant stuff anymore such as constraints
+		# containing fixed parameters etc. 
+		
+		# reparametrize for optim and nlm
+		noptpars=length(fitpars)
+		# un-constrained pars
+		uncon=rep(0,noptpars)
+		constr=rep(0,noptpars)
+		for(i in 1:noptpars) {
+			if(fixed[i]==1) {
+				if(sum(as.logical(xgmod$A[,i]))==0) uncon[i]=1
+				else constr[i]=1
+			}
+		}
+		
+		# remove cols corresponding with non-constrained and fixed pars
+		A=A[,which(constr==1),drop=FALSE]
+		
+		# remove rows correponding with inequality constraints
+		idx=which(bllin<bulin) 
+		ci=matrix(bulin,nc=1) 
+		ineq=FALSE
+		if(length(idx)>0) {
+			A=A[-idx,,drop=FALSE]
+			ci=ci[-idx]
+			Aineq=xgmod$A[idx,which(constr==1),drop=FALSE]
+			ciIneqLow=matrix(xgmod$bllin[idx],nc=1)
+			ciIneqUp=matrix(xgmod$bulin[idx],nc=1)
+			ineq=TRUE
+		}
+		
+		if(nrow(A)>0) {
+			b=Null(t(A))
+			ginvb=ginv(b)
+			ginvA=ginv(A)
+			newpars=ginvb%*%(fitpars[which(constr==1)]-ginvA%*%ci)
+			nnewpars=length(newpars)
+		} else {
+			nnewpars=0
+		}
+		
+		ucpars=xgmod$pars[which(uncon==1)]
+		nucpars=length(ucpars)
+		if(nnewpars>0) {
+			pars=c(newpars,ucpars)
+		} else {
+			pars=ucpars
+		}
+		
+		# bounds on original parameters = linear constraints on new parameters
+		bucon=bu[which(constr==1)]
+		blcon=bl[which(constr==1)]
+		uiOpt=rbind(-b,b)
+		ciOpt=c((-bucon+ginvA%*%ci),(blcon-ginvA%*%ci))
+		
+		if(nucpars>0) {
+			uiOpt=bdiag(list(uiOpt,diag(rep(0,nucpars))))
+			ciOpt=c(ciOpt,rep(-1e-10,nucpars))
+		}
+		
+		# linear inequality constraints on original parameters
+		if(ineq) {
+			uiOptLin=rbind(-Aineq%*%b,Aineq%*%b)
+			if(nucpars>0) {
+				zz=matrix(0,nrow(uiOptLin),nucpars)
+				uiOptLin=cbind(uiOptLin,zz)
+			}
+			ciOpt=c(ciOpt,Aineq%*%ginvA%*%ci-ciIneqUp,-Aineq%*%ginvA%*%ci+ciIneqLow)
+			uiOpt=rbind(uiOpt,uiOptLin)
+		}
+				
+		assign("fcalls",0,envir=.GlobalEnv)
+		
+		# gloptim optimization
+		f <- function(pars) {
+			# expand pars
+			if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
+			if(nucpars>0) xgmod$pars[which(uncon==1)]=pars[(nnewpars+1):(nnewpars+nucpars)]
+			ll=loglike(dat=dat,dmm=xgmod,print=0,set=FALSE,tdcov=tdcov)$logl
+			xgpars=xgmod$pars
+			assign("fcalls",fcalls+1,envir=.GlobalEnv)
+			if(is.nan(ll) || is.infinite(ll)) ll=initLogl*1.01
+			pr=0
+			if(printlevel>9 && (fcalls%%10)==0) {cat("call " , fcalls, " logl ", ll, "\n"); pr=1}
+			if(printlevel>14 && pr==0) cat("call " , fcalls, " logl ", ll, "\n")
+			ll
+		}
+		
+		gr <- function(pars) {
+			# expand pars
+			if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
+			if(nucpars>0) xgmod$pars[which(uncon==1)]=pars[(nnewpars+1):(nnewpars+nucpars)]
+			gr=loglike(dat=dat,dmm=xgmod,grad=TRUE,print=0,set=FALSE)$gr
+			if(nnewpars>0) grad=c(gr[which(constr==1)]%*%b,gr[which(uncon==1)])
+			else grad=gr[which(uncon==1)]
+			if(printlevel>0 && (fcalls%%10)==0) cat("call " , fcalls, " grad ", grad, "\n")
+			if(printlevel>9) cat("call " , fcalls, " grad ", grad, "\n")
+			grad
+		}
+		
+		# optim control parameters
+		nopt=1
+		fnscale=-1.0
+		trace=1
+		factr=1e7
+		maxit=5000
+		REPORT=10
+		if(printlevel>1 && printlevel<5) REPORT=5
+		if(printlevel>4) REPORT=1
+		if(printlevel==0) REPORT=0
+		control=list(fnscale=fnscale,trace=trace,factr=factr,maxit=maxit,REPORT=REPORT)
+		oldObjf=initLogl
+		
+		for(i in 1:nopt) { 
+			# this actually seems to be working just fine, but it does depend on mu a little bit
+			
+			if(der) z<-constrOptim(theta=pars,f,gr=gr,ui=uiOpt,ci=ciOpt,control=control,outer.iterations=500,mu=1e-04)
+			if(der==0) z<-constrOptim(theta=pars,f,gr=NULL,ui=uiOpt,ci=ciOpt,control=control,outer.iterations=500,mu=5e-01)
+			print(z)
+			if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
+			if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
+			
+			if(totalviolation > 1e-9) {
+				cat("Constraint violation not zero, minimum probably not optimal, provide better starting values.\n")
+				cat("Violation is: ", totalviolation, "\n")
+			}
+			
+			if(z$convergence==0 && totalviolation < 1e-9) break
+			violationPenalty=3*violationPenalty
+			oldObjf=z$value
+			pars=z$par
+		}
+		names(z)=c("par","objf","iter","inform","message")
+		
+		# put fitted pars into xgmod, gather output into z
+		if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
+		if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
+		
+		z$timeUsed=timeUsed
+		z$pars=xgmod$pars
+		z$npars=xgmod$npars
+		z$totMem=NULL
+		
+	} # end gloptim
+	
 ##  call optim or nlm to optimize the model
 	if(method=="optim" || method=="nlm") {
  	
@@ -321,67 +477,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		}
 				
 		assign("fcalls",0,envir=.GlobalEnv)
-		assign("totalviolation",0,envir=.GlobalEnv)
-		
-	# nlm optimization
-		if(method=="nlm") {
-			assign("loglvalue",0,envir=.GlobalEnv)
-			
-			f <- function(pars) {
-				# expand pars into the model
-   				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
-   				if(nucpars>0) xgmod$pars[which(uncon==1)]=pars[(nnewpars+1):(nnewpars+nucpars)]
-				# get the actual likelihood and gradients if requested/feasible
-				if(der) {
-					llgr=loglike(dat=dat,dmm=xgmod,print=0,grad=TRUE,set=FALSE)
-					if(nnewpars>0) grad=c(-llgr$gr[which(constr==1)]%*%b,-llgr$gr[which(uncon==1)])
-					else grad=-llgr$gr[which(uncon==1)]
-				} else llgr=loglike(dat=dat,dmm=xgmod,tdcov=tdcov,print=0,grad=FALSE,set=FALSE)
-				ll=llgr$logl
-				violation=viol(xgmod$pars)
-				assign("totalviolation",violation,envir=.GlobalEnv)
-				ll=ll-violation
-				pr=0
-				assign("fcalls",fcalls+1,envir=.GlobalEnv)
-				if(is.nan(ll) || is.infinite(ll)) ll=initLogl*1.01
-				if(printlevel>9 && (fcalls%%10)==0) {cat("call " , fcalls, " logl ", ll, "\n"); pr=1}
-				if(printlevel>14 && pr==0) cat("call " , fcalls, " logl ", ll, "\n")
-				if(der) attr(ll,'gradient')=grad
-				-ll
-			}
-			
-			prl=1
-			if(printlevel==0) prl=0
-			if(printlevel>4) prl=2
-			
-			ndig=12
-			if(accuracy=="high") ndig=15
-			if(accuracy=="best") ndig=18
-						
-			nopt=1
-			
-			adddig=3
-			
-			for(i in 1:nopt) {
-				gtol=10^-(ndig-6)
-				stol=10^-(ndig-6)
-				timeUsed=timeUsed+system.time(z<-nlm(f,p=pars,hessian=FALSE,ndigit=ndig,gradtol=gtol,steptol=stol,fscale=-initLogl*0.95,print.level=prl,check.analyticals = FALSE))[3]
-				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$estimate[1:nnewpars]
-				if(nucpars>0) xgmod$pars[which(uncon==1)]=z$estimate[(nnewpars+1):(nnewpars+nucpars)]
-				if(z$code<3 && totalviolation<1e-9) break
-				if(z$code==3) { ndig=ndig+adddig; vfactor=3*vfactor}
- 				if(totalviolation > 1e-9) vfactor=3*vfactor
-				if(z$code>3) break
-			}
-			
-			if(totalviolation > 1e-9) {
-				cat("Constraints not satisfies provide better starting/fixed values or increase vfactor.\n")
-				cat("Violation is: ", totalviolation, "\n")
-			}
-			
-			names(z)=c("objf","par","grad","inform","iter")
-			z$objf=ifelse(loglvalue==0,-z$objf,loglvalue)
-		} # end nlm
+		assign("totalviolation",0,envir=.GlobalEnv)		
 		
 	# optim optimization
 		if(method=="optim") {
@@ -408,15 +504,16 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 				if(nnewpars>0) grad=c(gr[which(constr==1)]%*%b,gr[which(uncon==1)])
 				else grad=gr[which(uncon==1)]
 				if(printlevel>0 && (fcalls%%10)==0) cat("call " , fcalls, " grad ", grad, "\n")
+				if(printlevel>9) cat("call " , fcalls, " grad ", grad, "\n")
 				grad
 			}
 			
 	# optim control parameters
-			nopt=3 
+			nopt=1
 			fnscale=-1.0
 			trace=1
 			factr=1e7
-			maxit=100
+			maxit=5000
 			REPORT=10
 			if(printlevel>1 && printlevel<5) REPORT=5
 			if(printlevel>4) REPORT=1
@@ -425,7 +522,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 			oldObjf=initLogl
 			
 			for(i in 1:nopt) { 
-				timeUsed=timeUsed+system.time(z<-optim(pars,f,gr=NULL,control=control,method="L-BFGS-B",hessian=FALSE))[3]
+  				timeUsed=timeUsed+system.time(z<-optim(pars,f,gr=NULL,control=control,method="BFGS",hessian=FALSE))[3]
 				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
 				if(nucpars>0) xgmod$pars[which(uncon==1)]=z$par[(nnewpars+1):(nnewpars+nucpars)]
 				
@@ -438,9 +535,67 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 				violationPenalty=3*violationPenalty
 				oldObjf=z$value
 				pars=z$par
+				
 			}
 			names(z)=c("par","objf","iter","inform","message")
 		} # end optim
+
+		# nlm optimization
+		if(method=="nlm") {
+			assign("loglvalue",0,envir=.GlobalEnv)
+			
+			f <- function(pars) {
+				# expand pars into the model
+				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%pars[1:nnewpars]
+				if(nucpars>0) xgmod$pars[which(uncon==1)]=pars[(nnewpars+1):(nnewpars+nucpars)]
+				# get the actual likelihood and gradients if requested/feasible
+				if(der) {
+					llgr=loglike(dat=dat,dmm=xgmod,print=0,grad=TRUE,set=FALSE)
+					if(nnewpars>0) grad=c(-llgr$gr[which(constr==1)]%*%b,-llgr$gr[which(uncon==1)])
+					else grad=-llgr$gr[which(uncon==1)]
+				} else llgr=loglike(dat=dat,dmm=xgmod,tdcov=tdcov,print=0,grad=FALSE,set=FALSE)
+				ll=llgr$logl
+				violation=viol(xgmod$pars)
+				assign("totalviolation",violation,envir=.GlobalEnv)
+				ll=ll-violation
+				pr=0
+				assign("fcalls",fcalls+1,envir=.GlobalEnv)
+				if(is.nan(ll) || is.infinite(ll)) ll=initLogl*1.01
+				if(printlevel>9 && (fcalls%%10)==0) {cat("call " , fcalls, " logl ", ll, "\n"); pr=1}
+				if(printlevel>14 && pr==0) cat("call " , fcalls, " logl ", ll, "\n")
+				if(der) attr(ll,'gradient')=grad
+				-ll
+			}
+			
+			prl=1
+			if(printlevel==0) prl=0
+			if(printlevel>4) prl=2
+			ndig=12
+			if(accuracy=="high") ndig=15
+			if(accuracy=="best") ndig=18
+			nopt=1
+			adddig=3
+			
+			for(i in 1:nopt) {
+				gtol=10^-(ndig-6)
+				stol=10^-(ndig-6)
+				timeUsed=timeUsed+system.time(z<-nlm(f,p=pars,hessian=FALSE,ndigit=ndig,gradtol=gtol,steptol=stol,fscale=-initLogl*0.95,print.level=prl,check.analyticals = FALSE))[3]
+				if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$estimate[1:nnewpars]
+				if(nucpars>0) xgmod$pars[which(uncon==1)]=z$estimate[(nnewpars+1):(nnewpars+nucpars)]
+				if(z$code<3 && totalviolation<1e-9) break
+				if(z$code==3) { ndig=ndig+adddig; vfactor=3*vfactor}
+				if(totalviolation > 1e-9) vfactor=3*vfactor
+				if(z$code>3) break
+			}
+			
+			if(totalviolation > 1e-9) {
+				cat("Constraints not satisfies provide better starting/fixed values or increase vfactor.\n")
+				cat("Violation is: ", totalviolation, "\n")
+			}
+			
+			names(z)=c("objf","par","grad","inform","iter")
+			z$objf=ifelse(loglvalue==0,-z$objf,loglvalue)
+		} # end nlm
 		
 	# put fitted pars into xgmod, gather output into z
 		if(nnewpars>0) xgmod$pars[which(constr==1)]=ginvA%*%ci + b%*%z$par[1:nnewpars]
@@ -450,7 +605,7 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		z$pars=xgmod$pars
 		z$npars=xgmod$npars
 		z$totMem=NULL
-	} # end optim or nlm optimization
+	} # end optim/nlm optimization
 	
 	if(printlevel>0) cat("Final loglikelihood: ", z$objf,"\n")
 	
@@ -526,9 +681,6 @@ fitdmm <- function(dat,dmm,printlevel=1,poster=TRUE,tdcov=0,ses=TRUE,method="nlm
 		df=mod$freeparsnotd
 	} else df=mod$freepars
 	aic = -2*z$objf+2*df
-	nobs1=0
-	if(xgmod$ng==1) nobs1=sum(ntimes(dat))
-	else for(i in 1:xgmod$ng) nobs1 = nobs1 + sum(ntimes(dat[[i]]))	
 	bic = -2*z$objf+log(nobs1)*df
 	
 	if(printlevel>0) {
@@ -740,7 +892,9 @@ bootstrap <- function(object, dat, samples=100, pvalonly=0, ...) {
 			PACKAGE="depmix")
 	}
 	for(i in 1:samples) {
+		
 		gen=generate(dmm=mod, nt=dat$ntimes)
+		
 		ll=loglike(dat=gen,dmm=mod,print=0)$logl
 		fit=fitdmm(dat=gen,dmm=mod,print=0,ses=0,post=0)
 		bootpars[i,]=fit$mod$pars
