@@ -23,10 +23,10 @@ void mixModelSetUp(int *ngroups, int *nrcomp, int *nstates, int *nitems, int *it
 void covSetUp(int *ngroups, int *nrcomp, int *nstates, int *nitems, int *itemtypes, double *pars, int *xm, int *print);
 
 void multiDataSetUp(int *ngroups, int *printlevel);
-void ngDataSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, int *xm, int *printlevel);
+void ngDataSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, double *weights, int *xm, int *printlevel);
 
 void multiCovSetUp(int *ngroups, int *printlevel);
-void ngCovSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, int *xm, int *printlevel);
+void ngCovSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, double *weights, int *xm, int *printlevel);
 
 // the loglikelihood function
 void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess, double *gr, double *hs, 
@@ -41,4 +41,3 @@ void setCrit(double *criter);
 } //end extern "C"
 
 #endif
-
